#include "bullet.hpp"

#include <cmath>
#include <cstdlib>
#include <gint/display.h>

// Constructor of the Bullet object (need a target (X,Y) )
Bullet::Bullet(float TargetX, float TargetY) {
  Power = 2;
  Speed = 1.0f;

  ToBeDeleted = false;

  float len = 0.0f;

  BulletX = (float)(DWIDTH / 2);
  BulletY = (float)(DHEIGHT / 2);

  Size = Power;

  DX = (TargetX - BulletX);
  DY = (TargetY - BulletY);

  len = sqrt(DX * DX + DY * DY);

  DX /= len / Speed;
  DY /= len / Speed;

  // the color must be chosen depending on the machine (C_BLACK for monochrome
  // or anyRGB color for Prizm)
#if (FX9860G)
  Color = C_BLACK;
#else
  Color = C_GREEN;
#endif
}

// Destructor (empty cause neither dynamic allocation nor sub-classes)
Bullet::~Bullet(void) {}

// Implementation of the rendering method
void Bullet::Render(void) {
  dcircle((int)BulletX, (int)BulletY, Size, Color, Color);
}

// Implementation of the update method
void Bullet::Update(void) {
  if (ToBeDeleted)
    return;

  BulletX += DX;
  BulletY += DY;

  if (BulletX < 0 || BulletX > DWIDTH || BulletY < 0 || BulletY > DHEIGHT)
    ToBeDeleted = true;
}

// Implementation of the method called when Enemy is hit by this bullet
void Bullet::HitDone(void) { ToBeDeleted = true; }