#include "enemy.hpp"

#include <cmath>
#include <cstdlib>
#include <gint/display.h>

Enemy::Enemy(void) {
  Life = 1 + (rand() % 4);
  Speed = 0.025f;

  ToBeDeleted = false;

  float len = 0.0f;
  float multiplier = 1.0f + (rand() % 5) / 2.0f;
  int angle = rand() % 360;

  len = multiplier *
        sqrt((DWIDTH / 2) * (DWIDTH / 2) + (DHEIGHT / 2) * (DHEIGHT / 2));

  EnemyX = (float)(len * cos(angle) + DWIDTH / 2);
  EnemyY = (float)(len * sin(angle) + DHEIGHT / 2);

  DX = (EnemyX - DWIDTH / 2);
  DY = (EnemyY - DHEIGHT / 2);

  DX /= len / Speed;
  DY /= len / Speed;

  // the color must be chosen depending on the machine (C_BLACK for monochrome
  // or any RGB color for Prizm)
#if (FX9860G)
  Color = C_BLACK;
  Size = Life * 2;
#else
  Color = C_BLUE;
  Size = Life * 6;
#endif
}

Enemy::~Enemy(void) {}

void Enemy::Render(void) {
  drect_border((int)(EnemyX - Size / 2), (int)(EnemyY - Size / 2),
               (int)(EnemyX + Size / 2), (int)(EnemyY + Size / 2), C_NONE, 2,
               Color);
}

void Enemy::Update(void) {
  if (ToBeDeleted)
    return;

  EnemyX -= DX;
  EnemyY -= DY;

  float ddX = (EnemyX - DWIDTH / 2);
  float ddY = (EnemyY - DHEIGHT / 2);
  float ddlen = (ddX * ddX + ddY * ddY);

  DistanceToTower = ddlen;

  if (ddlen <= DHEIGHT * DHEIGHT / 100) {
    DX = 0;
    DY = 0;
  }

  if (Life == 0)
    ToBeDeleted = true;
}

bool Enemy::TestImpact(Bullet *Shot) {
  if (Shot->BulletX <= EnemyX - Size / 2 ||
      Shot->BulletX >= EnemyX + Size / 2 ||
      Shot->BulletY <= EnemyY - Size / 2 || Shot->BulletY >= EnemyY + Size / 2)
    return false;

  if (Life >= Shot->Power)
    Life -= Shot->Power;
  else
    Life = 0;

  return true;
}